Imports System.Threading

Namespace BuisnessLogicLayer


  Public Class PeerChannel : Inherits PscChannel
    Implements IChannel

    Private _offset As Integer
    Protected _childChannels As New List(Of PeerChannel)

    Public Shadows Event PositionCommand As PositionCommandEventHandler
    Public Event OffsetChanged As EventHandler



#Region "Properties"

    Public Property Offset() As Integer
      Get
        Return _offset
      End Get
      Set(ByVal value As Integer)
        If _offset <> value Then
          Dim offs As New OffsetArgs(value, Me)
          _offset = value
          OnOffsetChanged(offs)
        End If
      End Set
    End Property 'Offset

    Public ReadOnly Property ChildChannels() As List(Of PeerChannel)
      Get
        Return _childChannels
      End Get
    End Property 'ChildChannels

    Public Property ChildChannel(ByVal index As Integer) As PeerChannel
      Get
        Return _childChannels(index)
      End Get
      Set(ByVal value As PeerChannel)
        _childChannels(index) = value
      End Set
    End Property 'ChildChannel

#End Region

#Region "Constructors"

    Public Sub New()
      MyBase.New()
      _offset = DefaultValues.OffsetMinMax
    End Sub 'New


    Public Sub New(ByVal Id As Integer)
      MyBase.New(Id)
      _offset = Offset
    End Sub 'New

    Public Sub New(ByVal channelId As Integer, ByVal max As Integer, ByVal min As Integer, ByVal position As Integer, ByVal ramp As Integer, ByVal inverse As Boolean, ByVal delay As Integer, ByVal grouped As Boolean)
      MyBase.New(channelId, max, min, position, ramp, inverse, delay, grouped)
      _offset = Offset
    End Sub 'New


    Public Sub New(ByVal channel As PscChannel, ByVal offset As Integer)
      MyBase.New(channel.ChannelId, channel.Max, channel.Min, channel.Position, channel.Ramp, channel.Inverse, channel.Delay, channel.Grouped)
      _offset = offset
    End Sub 'New

#End Region

    'Instance methods
    Public Sub Add(ByVal channel As PscChannel)
      Dim offset As Integer = channel.Position - Me.Position
      Dim newChannel As New PeerChannel(channel, offset)
      'Calculate new channel max and min
      newChannel.Max = Me.Max + newChannel.Offset
      newChannel.Min = Me.Min + newChannel.Offset
      _childChannels.Add(newChannel)
    End Sub 'channel


    'Static methods
    'TODO: Change the access method of these guys. Or use an interface?
    Public Shared Function CreateNewGroup(ByVal MasterChannel As PscChannel, ByVal ChildChannel As PscChannel) As PeerChannel
      Dim mChannel As New PeerChannel(MasterChannel, DefaultValues.OffsetMinMax)
      mChannel.Add(ChildChannel)
      Return mChannel
    End Function 'CreateNewGroup


    Public Shared Function CreateNewGroup(ByVal pscChannels As List(Of PscChannel)) As PeerChannel
      Dim mChannel As New PeerChannel(pscChannels(0), DefaultValues.OffsetMinMax)
      Dim i As Integer
      For i = 1 To pscChannels.Count - 1
        mChannel.Add(pscChannels(i))
      Next
      Return mChannel
    End Function 'CreateNewGroup


    Public Shared Function AddPeerChild(ByVal Group As PeerChannel, ByVal ChildChannel As PscChannel) As Integer
      Group.Add(ChildChannel)
      Return Group.ChildChannels.Count - 1
    End Function 'AddPeerChild


    Public Sub CalculateNewOffset(ByVal index As Integer, ByVal newPos As Integer)
      _childChannels(index).Offset = newPos - Me.position
    End Sub


    'Override Methods
    Public Overrides Sub Move()
      Dim e As New PscEventArgs(Me)
      RaiseEvent PositionCommand(Me, e)

      For Each channel As PeerChannel In _childChannels
        channel.position = Me.position + channel.Offset
        e = New PscEventArgs(channel)
        RaiseEvent PositionCommand(Me, e)
      Next
      Thread.Sleep(Me.Delay)
      MyBase.move()
    End Sub 'Move


    Public Overrides Sub OnRampChanged(ByVal e As EventArgs)
      'Set child ramp = parent ramp
      Dim ramp As Integer = Me.Ramp
      For Each ch As PeerChannel In _childChannels
        ch.Ramp = ramp
      Next
      'Call base event
      MyBase.OnRampChanged(e)
    End Sub 'OnRampChanged


    Public Overrides Sub OnMaxChanged(ByVal e As EventArgs)
      'Update child max bounds
      Dim max As Integer = Me.Max
      For Each ch As PeerChannel In _childChannels
        ch.Max = Me.Max + ch.Offset
      Next
      'Call the base class methods
      MyBase.OnMaxChanged(e)
    End Sub 'OnMaxChanged


    Public Overrides Sub OnMinChanged(ByVal e As EventArgs)
      'Update child min bounds
      Dim min As Integer = Me.Min
      For Each ch As PeerChannel In _childChannels
        ch.Min = Me.Min + ch.Offset
      Next
      'Call the base class methods
      MyBase.OnMinChanged(e)
    End Sub 'OnMinChanged


    Public Overrides Sub OnPositionChanged(ByVal e As System.EventArgs)
      Dim pos As Integer = Me.position
      For Each ch As PeerChannel In _childChannels
        ch.position = Me.position + ch.Offset
      Next
      MyBase.OnPositionChanged(e)
    End Sub 'OnPositionChanged


    Public Sub OnOffsetChanged(ByVal e As OffsetArgs)
      Me.Max = e.Max
      Me.Min = e.Min
      Me.position = e.Position
      RaiseEvent OffsetChanged(Me, e)
    End Sub 'OnOffsetChanged


    Public Overrides Function ToString() As String
      Dim returnString As String
      If Me.ChannelId >= 0 Then
        returnString = vbCrLf & _
                        "Channel:  " & Me.ChannelId & vbCrLf & _
                        "Max:      " & Me.Max & vbCrLf & _
                        "Min:      " & Me.Min & vbCrLf & _
                        "Position: " & Me.position & vbCrLf & _
                        "Ramp:     " & Me.Ramp & vbCrLf & _
                        "Offset:   " & Me.Offset & vbCrLf
      Else
        Return "Invalid Channel Number"
      End If

      Return returnString
    End Function 'ToString


  End Class 'PscChannelAbsoluteGroup




End Namespace



