Imports CrustCrawler.BuisnessLogicLayer
Imports System.Reflection


Public Class ChannelControl : Inherits UserControl

  Private WithEvents ICh As IChannel
  Private Channel As PscChannel
  Private prChannel As PeerChannel

  Private _changed As Boolean = False
  Private CancelUpdateEvent As Boolean = False
  Private drag As Boolean = False


  Public Property Changed() As Boolean
    Get
      Return _changed
    End Get
    Set(ByVal value As Boolean)
      _changed = value
    End Set
  End Property


  Public Sub BindChannel(ByVal DataSource As PscChannel)
    If DataSource Is Nothing Then
      Throw New ArgumentNullException("Assign a datasource before binding to " & Me.ToString)
    End If
    CancelUpdateEvent = True
    Select Case DataSource.GetType.Name
      Case "PscChannel"
        Channel = CType(DataSource, PscChannel)
        prChannel = Nothing
        ICh = Channel
      Case "PeerChannel"
        prChannel = CType(DataSource, PeerChannel)
        ICh = prChannel
        Channel = Nothing
    End Select
    UpdateUI()
    CancelUpdateEvent = False
  End Sub


  Private Sub UpdateUI()
    Me.txtPosition.Text = CStr(ICh.Position)
    Me.Slider.Value = CStr(ICh.Position)
    Me.lblPosition.Text = CStr(ICh.Position)
    Me.txtRamp.Text = CStr(ICh.Ramp)
    Me.txtRamp2.Text = CStr(ICh.Ramp)
    Me.txtDelay.Text = CStr(ICh.Delay)
    Me.txtDelay2.Text = CStr(ICh.Delay)
  End Sub


  Public Overrides Function ToString() As String
    Return Me.Name
  End Function

  Private Sub Slider_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Slider.MouseDown
    drag = True
  End Sub

  Private Sub Slider_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Slider.MouseUp
    drag = False
    If Not Me.chkActive.Checked Then
      ICh.move()
    End If
  End Sub


  Private Sub Slider_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider.Scroll
    ICh.Position = Me.Slider.Value
    If Me.chkActive.Checked Then
      ICh.move()
    End If
    Me.lblPosition.Text = Me.Slider.Value
    Me.lblPosition.Update()
    Me.txtPosition.Text = CStr(Me.Slider.Value)
  End Sub


  Private Sub ICh_ChannelChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.ChannelChanged
    _changed = True
  End Sub


  Private Sub ICh_DelayChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.DelayChanged
    Dim delay As Integer = CType(sender, IChannel).Delay
    Me.txtDelay.Text = CStr(delay)
    Me.txtDelay2.Text = CStr(delay)
    Me.txtDelay.Update()
    Me.txtDelay2.Update()
  End Sub


  Private Sub ICh_MaxChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MaxChanged
    Dim max As Integer = CType(sender, IChannel).Max
    If Me.Slider.Value > max Then
      Me.Slider.Maximum = max
      Me.Slider.Value = max
    Else
      Me.Slider.Maximum = max
    End If
  End Sub


  Private Sub GetTicks()
    Dim ticks As Integer = Me.Slider.Maximum - Me.Slider.Minimum / 2
    Me.Slider.TickFrequency = ticks
  End Sub


  Private Sub ICh_MinChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MinChanged
    Dim min As Integer = CType(sender, IChannel).Min
    If Me.Slider.Value < min Then
      Me.Slider.Value = min
      Me.Slider.Minimum = min
    Else
      Me.Slider.Minimum = min
    End If
    'GetTicks()
  End Sub

  Private Sub ICh_PositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.PositionChanged
    Dim pos As Integer

    If ICh.Inverse Then
      pos = Globals.Inverse(ICh.Max, ICh.Min, ICh.Position)
    Else
      pos = ICh.Position
    End If

    Me.txtPosition.Text = CStr(pos)
    Me.txtPosition.Update()
    Me.Slider.Value = CStr(pos)
    Me.lblPosition.Text = CStr(pos)
    Me.lblPosition.Update()

    'If Me.chkActive.Checked Then
    '  ICh.move()
    'End If
  End Sub


  Private Sub ICh_PositionCommand(ByVal sender As Object, ByVal e As BuisnessLogicLayer.PscEventArgs) Handles ICh.PositionCommand
    Dim pos As Integer

    If ICh.Inverse Then
      pos = Globals.Inverse(ICh.Max, ICh.Min, ICh.Position)
    Else
      pos = ICh.Position
    End If

    Me.txtPosition.Text = CStr(pos)
    Me.txtPosition.Update()
    Me.Slider.Value = CStr(pos)
    Me.lblPosition.Text = CStr(pos)
    Me.lblPosition.Update()

    'If Me.chkActive.Checked Then
    '  ICh.move()
    'End If

  End Sub


  Private Sub IChnl_RampChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.RampChanged
    Dim newRamp As Integer = CType(sender, IChannel).Ramp
    Me.txtRamp.Text = CStr(newRamp)
    Me.txtRamp.Update()
    Me.txtRamp2.Text = CStr(newRamp)
    Me.txtRamp2.Update()
  End Sub


  Private Sub PostionTextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtPosition.LostFocus
    ICh.Position = CInt(CType(sender, TextBox).Text)
  End Sub


  Private Sub RampChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtRamp.LostFocus, txtRamp2.LostFocus
    ICh.Ramp = CInt(CType(sender, TextBox).Text)
  End Sub


  Private Sub DelayChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtDelay.LostFocus, txtDelay2.LostFocus
    ICh.Delay = CInt(CType(sender, TextBox).Text)
  End Sub


  Private Sub btnMove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMove.Click
    ICh.Position = CInt(Me.txtPosition.Text)
    ICh.Delay = CInt(Me.txtDelay.Text)
    ICh.Ramp = CInt(Me.txtRamp.Text)

    If _changed Then
      ICh.move()
    End If

    _changed = False
  End Sub


  Private Sub tsbProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbProperties.Click

    If Not Channel Is Nothing Then
      Dim propForm As New ChannelPropertySheet(Me.Channel)
      propForm.Show()
    End If

    If Not prChannel Is Nothing Then
      Dim propForm As New ChannelPropertySheet(Me.prChannel)
      propForm.Show()
    End If
  End Sub

End Class 'ChannelControl

